@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2015-2021 Dmitry V. Levin <ldv@strace.io>
+ * Copyright (c) 2021-2023 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
@@ -20,7 +21,7 @@ print_sigmask_addr_size(const void *const addr, const unsigned int size)
 {
 	tprint_struct_begin();
 	tprints_field_name("mask");
-	tprints(sprintsigmask_n("", addr, size));
+	tprints_string(sprintsigmask_n("", addr, size));
 	tprint_struct_end();
 }
 
