@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2022 The strace developers.
+ * Copyright (c) 2017-2024 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
@@ -48,7 +48,13 @@ typedef struct {
 	uint32_t stx_dio_mem_align; /* Memory buffer alignment for direct I/O */
 	uint32_t stx_dio_offset_align; /* File offset alignment for direct I/O */
 
-	uint64_t reserved2[12]; /* Spare space for future expansion */
+	uint64_t stx_subvol; /* Subvolume identifier */
+	uint32_t stx_atomic_write_unit_min; /* Min atomic write unit in bytes */
+	uint32_t stx_atomic_write_unit_max; /* Max atomic write unit in bytes */
+	uint32_t stx_atomic_write_segments_max; /* Max atomic write segment count */
+
+	uint32_t reserved1[1];
+	uint64_t reserved2[9]; /* Spare space for future expansion */
 } struct_statx;
 
 #endif /* !STRACE_STATX_H */
