@@ -3,21 +3,18 @@
  *
  * Copyright (c) 2016 JingPiao Chen <chenjingpiao@foxmail.com>
  * Copyright (c) 2016 Eugene Syromyatnikov <evgsyr@gmail.com>
- * Copyright (c) 2016-2018 The strace developers.
+ * Copyright (c) 2016-2021 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "tests.h"
-#include <asm/unistd.h>
+#include "scno.h"
+#include <stdio.h>
+#include <unistd.h>
 #include <linux/prctl.h>
 
-#if defined __NR_prctl && defined PR_GET_TSC && defined PR_SET_TSC
-
-# include <stdio.h>
-# include <unistd.h>
-
 int
 main(void)
 {
@@ -27,6 +24,8 @@ main(void)
 	TAIL_ALLOC_OBJECT_CONST_PTR(int, tsc);
 	long rc;
 
+	prctl_marker();
+
 	rc = syscall(__NR_prctl, PR_SET_TSC, 0);
 	printf("prctl(PR_SET_TSC, 0 /* PR_TSC_??? */) = %s\n", sprintrc(rc));
 
@@ -53,9 +52,3 @@ main(void)
 	puts("+++ exited with 0 +++");
 	return 0;
 }
-
-#else
-
-SKIP_MAIN_UNDEFINED("__NR_prctl && PR_GET_TSC && PR_SET_TSC")
-
-#endif
