@@ -1,29 +1,9 @@
 /*
- * Copyright (c) 2015-2016 Dmitry V. Levin <ldv@altlinux.org>
- * Copyright (c) 2015-2018 The strace developers.
+ * Copyright (c) 2015-2016 Dmitry V. Levin <ldv@strace.io>
+ * Copyright (c) 2015-2024 The strace developers.
  * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. The name of the author may not be used to endorse or promote products
- *    derived from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include <stdio.h>
@@ -32,15 +12,23 @@
 #include <string.h>
 #include <unistd.h>
 #include <assert.h>
-#include "flock.h"
+#include <linux/fcntl.h>
+#include "pidns.h"
+#include "scno.h"
 
 #define FILE_LEN 4096
 
 #define TEST_FLOCK_EINVAL(cmd) test_flock_einval(cmd, #cmd)
 #define TEST_FLOCK64_EINVAL(cmd) test_flock64_einval(cmd, #cmd)
 
+#ifndef NEED_TEST_FLOCK64_EINVAL
+# if defined F_OFD_GETLK && defined F_OFD_SETLK && defined F_OFD_SETLKW
+#  define NEED_TEST_FLOCK64_EINVAL
+# endif
+#endif
+
 #ifdef HAVE_TYPEOF
-# define TYPEOF_FLOCK_OFF_T typeof(((struct_kernel_flock *) NULL)->l_len)
+# define TYPEOF_FLOCK_OFF_T typeof(((struct flock *) NULL)->l_len)
 #else
 # define TYPEOF_FLOCK_OFF_T off_t
 #endif
@@ -61,46 +49,48 @@ invoke_test_syscall(const unsigned int fd, const unsigned int cmd, void *const p
 static void
 test_flock_einval(const int cmd, const char *name)
 {
-	TAIL_ALLOC_OBJECT_CONST_PTR(struct_kernel_flock, fl);
+	TAIL_ALLOC_OBJECT_CONST_PTR(struct flock, fl);
 	memset(fl, 0, sizeof(*fl));
 	fl->l_type = F_RDLCK;
 	fl->l_start = (TYPEOF_FLOCK_OFF_T) 0xdefaced1facefeedULL;
 	fl->l_len = (TYPEOF_FLOCK_OFF_T) 0xdefaced2cafef00dULL;
 
 	invoke_test_syscall(0, cmd, fl);
+	pidns_print_leader();
 	printf("%s(0, %s, {l_type=F_RDLCK, l_whence=SEEK_SET"
 	       ", l_start=%jd, l_len=%jd}) = %s\n", TEST_SYSCALL_STR, name,
 	       (intmax_t) fl->l_start, (intmax_t) fl->l_len, errstr);
 
 	void *const bad_addr = (void *) fl + 1;
 	invoke_test_syscall(0, cmd, bad_addr);
+	pidns_print_leader();
 	printf("%s(0, %s, %p) = %s\n",
 	       TEST_SYSCALL_STR, name, bad_addr, errstr);
 }
 
-/*
- * This function is not declared static to avoid potential
- * "defined but not used" warning when included by fcntl.c
- */
-void
+#ifdef NEED_TEST_FLOCK64_EINVAL
+static void
 test_flock64_einval(const int cmd, const char *name)
 {
-	TAIL_ALLOC_OBJECT_CONST_PTR(struct_kernel_flock64, fl);
+	TAIL_ALLOC_OBJECT_CONST_PTR(struct flock64, fl);
 	memset(fl, 0, sizeof(*fl));
 	fl->l_type = F_RDLCK;
 	fl->l_start = (TYPEOF_FLOCK_OFF_T) 0xdefaced1facefeedULL;
 	fl->l_len = (TYPEOF_FLOCK_OFF_T) 0xdefaced2cafef00dULL;
 
 	invoke_test_syscall(0, cmd, fl);
+	pidns_print_leader();
 	printf("%s(0, %s, {l_type=F_RDLCK, l_whence=SEEK_SET"
 	       ", l_start=%jd, l_len=%jd}) = %s\n", TEST_SYSCALL_STR, name,
 	       (intmax_t) fl->l_start, (intmax_t) fl->l_len, errstr);
 
 	void *const bad_addr = (void *) fl + 1;
 	invoke_test_syscall(0, cmd, bad_addr);
+	pidns_print_leader();
 	printf("%s(0, %s, %p) = %s\n",
 	       TEST_SYSCALL_STR, name, bad_addr, errstr);
 }
+#endif /* NEED_TEST_FLOCK64_EINVAL */
 
 static void
 test_flock(void)
@@ -108,12 +98,13 @@ test_flock(void)
 	TEST_FLOCK_EINVAL(F_SETLK);
 	TEST_FLOCK_EINVAL(F_SETLKW);
 
-	TAIL_ALLOC_OBJECT_CONST_PTR(struct_kernel_flock, fl);
+	TAIL_ALLOC_OBJECT_CONST_PTR(struct flock, fl);
 	memset(fl, 0, sizeof(*fl));
 	fl->l_type = F_RDLCK;
 	fl->l_len = FILE_LEN;
 
 	long rc = invoke_test_syscall(0, F_SETLK, fl);
+	pidns_print_leader();
 	printf("%s(0, F_SETLK, {l_type=F_RDLCK, l_whence=SEEK_SET"
 	       ", l_start=0, l_len=%d}) = %s\n",
 	       TEST_SYSCALL_STR, FILE_LEN, errstr);
@@ -121,11 +112,13 @@ test_flock(void)
 		return;
 
 	invoke_test_syscall(0, F_GETLK, fl);
+	pidns_print_leader();
 	printf("%s(0, F_GETLK, {l_type=F_UNLCK, l_whence=SEEK_SET"
 	       ", l_start=0, l_len=%d, l_pid=0}) = 0\n",
 	       TEST_SYSCALL_STR, FILE_LEN);
 
 	invoke_test_syscall(0, F_SETLKW, fl);
+	pidns_print_leader();
 	printf("%s(0, F_SETLKW, {l_type=F_UNLCK, l_whence=SEEK_SET"
 	       ", l_start=0, l_len=%d}) = 0\n",
 	       TEST_SYSCALL_STR, FILE_LEN);
@@ -138,12 +131,13 @@ test_flock64_ofd(void)
 	TEST_FLOCK64_EINVAL(F_OFD_SETLK);
 	TEST_FLOCK64_EINVAL(F_OFD_SETLKW);
 
-	TAIL_ALLOC_OBJECT_CONST_PTR(struct_kernel_flock64, fl);
+	TAIL_ALLOC_OBJECT_CONST_PTR(struct flock64, fl);
 	memset(fl, 0, sizeof(*fl));
 	fl->l_type = F_RDLCK;
 	fl->l_len = FILE_LEN;
 
 	long rc = invoke_test_syscall(0, F_OFD_SETLK, fl);
+	pidns_print_leader();
 	printf("%s(0, F_OFD_SETLK, {l_type=F_RDLCK, l_whence=SEEK_SET"
 	       ", l_start=0, l_len=%d}) = %s\n",
 	       TEST_SYSCALL_STR, FILE_LEN, errstr);
@@ -151,11 +145,13 @@ test_flock64_ofd(void)
 		return;
 
 	invoke_test_syscall(0, F_OFD_GETLK, fl);
+	pidns_print_leader();
 	printf("%s(0, F_OFD_GETLK, {l_type=F_UNLCK, l_whence=SEEK_SET"
 	       ", l_start=0, l_len=%d, l_pid=0}) = 0\n",
 	       TEST_SYSCALL_STR, FILE_LEN);
 
 	invoke_test_syscall(0, F_OFD_SETLKW, fl);
+	pidns_print_leader();
 	printf("%s(0, F_OFD_SETLKW, {l_type=F_UNLCK, l_whence=SEEK_SET"
 	       ", l_start=0, l_len=%d}) = 0\n",
 	       TEST_SYSCALL_STR, FILE_LEN);
@@ -171,34 +167,24 @@ test_flock64(void)
 	test_flock64_lk64();
 }
 
-/*
- * F_[GS]ETOWN_EX had conflicting values with F_[SG]ETLK64
- * in kernel revisions v2.6.32-rc1~96..v2.6.32-rc7~23.
- */
-#undef TEST_F_OWNER_EX
-#if defined F_GETOWN_EX && defined F_SETOWN_EX \
- && (F_GETOWN_EX != F_SETLK64) && (F_SETOWN_EX != F_GETLK64)
-# define TEST_F_OWNER_EX
-#endif
-
-#ifdef TEST_F_OWNER_EX
-# include "f_owner_ex.h"
-
 static long
 test_f_owner_ex_type_pid(const int cmd, const char *const cmd_name,
 			 const int type, const char *const type_name,
-			 pid_t pid)
+			 enum pid_type pid_type, pid_t pid)
 {
-	TAIL_ALLOC_OBJECT_CONST_PTR(struct_kernel_f_owner_ex, fo);
+	TAIL_ALLOC_OBJECT_CONST_PTR(struct f_owner_ex, fo);
 
 	fo->type = type;
 	fo->pid = pid;
 	long rc = invoke_test_syscall(0, cmd, fo);
-	printf("%s(0, %s, {type=%s, pid=%d}) = %s\n",
-	       TEST_SYSCALL_STR, cmd_name, type_name, fo->pid, errstr);
+	pidns_print_leader();
+	printf("%s(0, %s, {type=%s, pid=%d%s}) = %s\n",
+	       TEST_SYSCALL_STR, cmd_name, type_name,
+	       fo->pid, pidns_pid2str(pid_type), errstr);
 
 	void *bad_addr = (void *) fo + 1;
 	invoke_test_syscall(0, cmd, bad_addr);
+	pidns_print_leader();
 	printf("%s(0, %s, %p) = %s\n",
 	       TEST_SYSCALL_STR, cmd_name, bad_addr, errstr);
 
@@ -207,38 +193,36 @@ test_f_owner_ex_type_pid(const int cmd, const char *const cmd_name,
 
 static void
 test_f_owner_ex_umove_or_printaddr(const int type, const char *const type_name,
-				   pid_t pid)
+				   enum pid_type pid_type, pid_t pid)
 {
 	long rc = test_f_owner_ex_type_pid(ARG_STR(F_SETOWN_EX),
-					   type, type_name, pid);
+					   type, type_name, pid_type, pid);
 	if (!rc)
 		test_f_owner_ex_type_pid(ARG_STR(F_GETOWN_EX),
-					 type, type_name, pid);
+					 type, type_name, pid_type, pid);
 }
 
 static void
 test_f_owner_ex(void)
 {
-	static const struct {
+	struct {
 		int type;
 		const char *type_name;
-		pid_t pid[2];
+		enum pid_type pid_type;
+		pid_t pid;
 	} a[] = {
-		{ ARG_STR(F_OWNER_TID), { 1234567890, 20 } },
-		{ ARG_STR(F_OWNER_PID), { 1298126790, 30 } },
-		{ ARG_STR(F_OWNER_PGRP), { 1294567890, 40 } }
+		{ ARG_STR(F_OWNER_TID), PT_NONE, 1234567890 },
+		{ ARG_STR(F_OWNER_PID), PT_NONE, 1234567890 },
+		{ ARG_STR(F_OWNER_PGRP), PT_NONE, 1234567890 },
+		{ ARG_STR(F_OWNER_TID), PT_TID, syscall(__NR_gettid) },
+		{ ARG_STR(F_OWNER_PID), PT_TGID, getpid() },
+		{ ARG_STR(F_OWNER_PGRP), PT_PGID, getpgid(0) },
 	};
 
-	for (unsigned int i = 0; i < ARRAY_SIZE(a); i++) {
-		for (unsigned int j = 0; j < ARRAY_SIZE(a[0].pid); j++) {
-			test_f_owner_ex_umove_or_printaddr(a[i].type,
-							   a[i].type_name,
-							   a[i].pid[j]);
-		}
-	}
+	for (unsigned int i = 0; i < ARRAY_SIZE(a); i++)
+		test_f_owner_ex_umove_or_printaddr(a[i].type, a[i].type_name,
+			a[i].pid_type, a[i].pid);
 }
-#endif /* TEST_F_OWNER_EX */
-
 struct fcntl_cmd_check {
 	int fd;
 	int cmd;
@@ -248,6 +232,23 @@ struct fcntl_cmd_check {
 	void (*print_flags)(long rc);
 };
 
+static void
+test_xetown(void)
+{
+	const int pid = getpid();
+	const char *pid_str = pidns_pid2str(PT_TGID);
+
+	invoke_test_syscall(0, F_SETOWN, (void *) (intptr_t) pid);
+	pidns_print_leader();
+	printf("%s(0, F_SETOWN, %d%s) = %s\n",
+		TEST_SYSCALL_STR, pid, pid_str, errstr);
+
+	invoke_test_syscall(0, F_GETOWN, NULL);
+	pidns_print_leader();
+	printf("%s(0, F_GETOWN) = %d%s\n",
+		TEST_SYSCALL_STR, pid, pid_str);
+}
+
 static void
 print_retval_flags(const struct fcntl_cmd_check *check, long rc)
 {
@@ -263,12 +264,14 @@ static void
 test_other_set_cmd(const struct fcntl_cmd_check *check)
 {
 	invoke_test_syscall(check->fd, check->cmd, (void *) check->arg);
+	pidns_print_leader();
 	printf("%s(%d, %s, %s) = %s\n",
 	       TEST_SYSCALL_STR, check->fd,
 	       check->cmd_str, check->arg_str, errstr);
 
 	/* bad file fd */
 	invoke_test_syscall(-1, check->cmd, (void *) check->arg);
+	pidns_print_leader();
 	printf("%s(-1, %s, %s) = %s\n",
 	       TEST_SYSCALL_STR, check->cmd_str,
 	       check->arg_str, errstr);
@@ -278,12 +281,14 @@ static void
 test_other_get_cmd(const struct fcntl_cmd_check *check)
 {
 	long rc = invoke_test_syscall(check->fd, check->cmd, NULL);
+	pidns_print_leader();
 	printf("%s(%d, %s) = ",
 	       TEST_SYSCALL_STR, check->fd, check->cmd_str);
 	print_retval_flags(check, rc);
 
 	/* bad file fd */
 	invoke_test_syscall(-1, check->cmd, NULL);
+	pidns_print_leader();
 	printf("%s(-1, %s) = %s\n",
 	       TEST_SYSCALL_STR, check->cmd_str, errstr);
 }
@@ -335,18 +340,14 @@ test_fcntl_others(void)
 {
 	static const struct fcntl_cmd_check set_checks[] = {
 		{ 0, ARG_STR(F_SETFD), ARG_STR(FD_CLOEXEC) },
-		{ 0, ARG_STR(F_SETOWN), ARG_STR(20) },
-#ifdef F_SETPIPE_SZ
 		{ 0, ARG_STR(F_SETPIPE_SZ), ARG_STR(4097) },
-#endif
 		{ 0, ARG_STR(F_DUPFD), ARG_STR(0) },
-#ifdef F_DUPFD_CLOEXEC
 		{ 0, ARG_STR(F_DUPFD_CLOEXEC), ARG_STR(0) },
-#endif
 		{ 0, ARG_STR(F_SETFL), ARG_STR(O_RDWR|O_LARGEFILE) },
 		{ 0, ARG_STR(F_NOTIFY), ARG_STR(DN_ACCESS) },
+		{ 0, ARG_STR(F_DUPFD_QUERY), ARG_STR(0) },
 		{ 1, ARG_STR(F_SETLEASE), ARG_STR(F_RDLCK) },
-		{ 0, ARG_STR(F_SETSIG), 0, "SIG_0" },
+		{ 0, ARG_STR(F_SETSIG), 0, "0" },
 		{ 1, ARG_STR(F_SETSIG), 1, "SIGHUP" }
 	};
 	for (unsigned int i = 0; i < ARRAY_SIZE(set_checks); i++) {
@@ -356,10 +357,7 @@ test_fcntl_others(void)
 	static const struct fcntl_cmd_check get_checks[] = {
 		{ 0, ARG_STR(F_GETFD), .print_flags = print_flags_getfd },
 		{ 1, ARG_STR(F_GETFD), .print_flags = print_flags_getfd },
-		{ 0, ARG_STR(F_GETOWN) },
-#ifdef F_GETPIPE_SZ
 		{ 0, ARG_STR(F_GETPIPE_SZ) },
-#endif
 		{ 1, ARG_STR(F_GETLEASE), .print_flags = print_flags_getlease },
 		{ 0, ARG_STR(F_GETSIG), .print_flags = print_flags_getsig },
 		{ 1, ARG_STR(F_GETSIG), .print_flags = print_flags_getsig }
@@ -372,28 +370,24 @@ test_fcntl_others(void)
 static void
 create_sample(void)
 {
-	char fname[] = TEST_SYSCALL_STR "_XXXXXX";
-
 	(void) close(0);
-	if (mkstemp(fname))
-		perror_msg_and_fail("mkstemp: %s", fname);
-	if (unlink(fname))
-		perror_msg_and_fail("unlink: %s", fname);
-	if (ftruncate(0, FILE_LEN))
+	if (ftruncate(create_tmpfile(O_RDWR), FILE_LEN))
 		perror_msg_and_fail("ftruncate");
 }
 
 int
 main(void)
 {
+	PIDNS_TEST_INIT;
+
 	create_sample();
 	test_flock();
 	test_flock64();
-#ifdef TEST_F_OWNER_EX
 	test_f_owner_ex();
-#endif
 	test_fcntl_others();
+	test_xetown();
 
+	pidns_print_leader();
 	puts("+++ exited with 0 +++");
 	return 0;
 }
