@@ -2,7 +2,7 @@
  * Check decoding of PTP_* commands of ioctl syscall.
  *
  * Copyright (c) 2018 Harsha Sharma <harshasharmaiitr@gmail.com>
- * Copyright (c) 2018-2022 The strace developers.
+ * Copyright (c) 2018-2024 The strace developers.
  * All rights reserved.
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
@@ -100,7 +100,7 @@ test_no_device(void)
 	long rc;
 
 	/* unrecognized */
-	static const uint8_t unk_nums[] = { 0, 19, 20, 255 };
+	static const uint8_t unk_nums[] = { 0, 21, 22, 255 };
 
 	for (const uint8_t *p = unk_nums; p < ARRAY_END(unk_nums); p++) {
 		for (uint16_t sz = 0; sz < 1280; sz += 8) {
@@ -151,7 +151,25 @@ test_no_device(void)
 		if (rc >= 0) {
 			printf("{max_adj=0, n_alarm=0, n_ext_ts=0, n_per_out=0"
 			       ", pps=0, n_pins=0, cross_timestamping=0"
-			       ", adjust_phase=0}");
+			       ", adjust_phase=0, max_phase_adj=0}");
+		} else {
+			printf("%p", caps);
+		}
+		printf(") = %s\n", errstr);
+
+		fill_memory32_ex(caps, sizeof(*caps), 1000000, 1);
+		rc = sys_ioctl(-1, c->val, (uintptr_t) caps);
+		printf("ioctl(-1, " XLAT_FMT ", ", XLAT_SEL(c->val, c->str));
+		if (rc >= 0) {
+			printf("{max_adj=1000000, n_alarm=1000000"
+			       ", n_ext_ts=1000000, n_per_out=1000000"
+			       ", pps=1000000, n_pins=1000000"
+			       ", cross_timestamping=1000000"
+			       ", adjust_phase=1000000, max_phase_adj=1000000"
+			       NRAW(" /* 0.001000000 s */")
+			       ", rsv=[0xf4240, 0xf4240, 0xf4240"
+			       ", 0xf4240, 0xf4240, 0xf4240, 0xf4240"
+			       ", 0xf4240, 0xf4240, 0xf4240, 0xf4240]}");
 		} else {
 			printf("%p", caps);
 		}
@@ -166,7 +184,9 @@ test_no_device(void)
 			       ", pps=-2136948508, n_pins=-2136948507"
 			       ", cross_timestamping=-2136948506"
 			       ", adjust_phase=-2136948505"
-			       ", rsv=[0x80a0c0e8, 0x80a0c0e9, 0x80a0c0ea"
+			       ", max_phase_adj=-2136948504"
+			       NRAW(" /* -2.136948504 s */")
+			       ", rsv=[0x80a0c0e9, 0x80a0c0ea"
 			       ", 0x80a0c0eb, 0x80a0c0ec, 0x80a0c0ed"
 			       ", 0x80a0c0ee, 0x80a0c0ef, 0x80a0c0f0"
 			       ", 0x80a0c0f1, 0x80a0c0f2, 0x80a0c0f3]}");
@@ -184,8 +204,9 @@ test_no_device(void)
 	static const struct strval32 extts_flags[] = {
 		{ ARG_XLAT_KNOWN(0x1, "PTP_ENABLE_FEATURE") },
 		{ ARG_XLAT_KNOWN(0xdeadbabe, "PTP_RISING_EDGE|PTP_FALLING_EDGE"
-					     "|PTP_STRICT_FLAGS|0xdeadbab0") },
-		{ ARG_XLAT_UNKNOWN(0xbadbeef0, "PTP_???") },
+					     "|PTP_STRICT_FLAGS|PTP_EXT_OFFSET"
+					     "|0xdeadbaa0") },
+		{ ARG_XLAT_UNKNOWN(0xbadbeee0, "PTP_???") },
 		{ ARG_STR(0) },
 	};
 	for (const struct strval32 *c = ioc_extts; c < ARRAY_END(ioc_extts);
